#!/bin/bash
###############################################################################
# Health Checking Mechanism for existing units
###############################################################################

MOTD=/etc/motd

# Installation target directories
INSTALL_DIR=/usr/sbin/
HEALTHCHECK_SCRIPT=${INSTALL_DIR}/healthcheck.sh

SYSTEM_LOG=/home/spectracom/log/system.log

# Temporary patch directory
PATCH_DIR=/opt/upgrade/temp

host_name="$(/bin/hostname)"
log_format="$(/bin/date +"%b %d %T") ${host_name} ${host_name}: [hotpatch]"

if [[ -e $MOTD ]] ; then
    cat $MOTD | grep -qE "Version (5[.]1[.][2-9A-Z])|(5[.][2-7][.].)|(5[.]8[.][0-4dDeEgGjJpP])"
    if [[ "$?" -ne "0" ]] ; then
        echo "${log_format} version not supported" >> ${SYSTEM_LOG}
        exit 1 # A 5.x.x version was not found
    fi
else
    # If /etc/motd does not exist this is a unrecoverable failure
    # return the unit to the factory
    echo "${log_format} WARNING: Could not determine system version" >> ${SYSTEM_LOG}
    exit 1
fi

# Install scripts that come with the patch:
# We don't care about existing iterations of these scripts

mv $PATCH_DIR/smartctl ${INSTALL_DIR}/
mv $PATCH_DIR/healthcheck.sh ${INSTALL_DIR}/

chown root:root ${INSTALL_DIR}/smartctl
chown root:root ${INSTALL_DIR}/healthcheck.sh

chmod 755 ${INSTALL_DIR}/smartctl
chmod 711 ${INSTALL_DIR}/healthcheck.sh

# Install health check in sudoers file, if necessary

SUDOERS_FILE=/etc/sudoers
HEALTH_SUDOERS="spadmin  ALL = NOPASSWD: /usr/sbin/healthcheck.sh"

if ! grep -q "${HEALTH_SUDOERS}" ${SUDOERS_FILE} ; then
    echo "${HEALTH_SUDOERS}" | sudo EDITOR='tee -a' visudo
fi

# Create alias for healthcheck command, if not already present
PROFILE_FILE=/etc/profile
HEALTH_ALIAS="alias healthcheck.sh='sudo healthcheck.sh'"

if ! grep -q "${HEALTH_ALIAS}" ${PROFILE_FILE} ; then
    sed -i "/unset sh/i ${HEALTH_ALIAS}" ${PROFILE_FILE}
fi

/bin/sync
${HEALTHCHECK_SCRIPT} >> ${SYSTEM_LOG}

ret_code=$?
# If the health check is compatible with the unit, then proceed
if [ ${ret_code} -eq 0 ] ; then
    # Install cron job to issue health check daily
    if ! crontab -l | grep -i "${HEALTHCHECK_SCRIPT}" ; then
        (crontab -l; echo "@daily ${HEALTHCHECK_SCRIPT}") | crontab -
    fi

    # Replace diskhealth element:
    diskhealth_file="${PATCH_DIR}/diskhealth.smart"
    diskhealth_element=/srv/www2/app/View/Elements/diskhealth.ctp
    chown spui:root ${diskhealth_file}
    chmod 664 ${diskhealth_file}
    cp ${diskhealth_file} ${diskhealth_element}
    
    diskhealth_pane='echo $this->element( '\''diskhealth'\'' );'
    upgrades_page=/srv/www2/app/View/Upgrades/index.ctp
    
    # Determine if we need to inject the code into the web ui
    if ! grep -iq "diskhealth" ${upgrades_page} ; then
        inject_line=$(grep -n -m 1 '$this->end()' ${upgrades_page} | cut -d":" -f 1)
        sed -i "${inject_line}i${diskhealth_pane}" ${upgrades_page}
    fi
    exit 0
fi


# If western digital copy a special web ui message
if [ ${ret_code} -eq 4 ] ; then
    # Replace diskhealth element:
    diskhealth_file="${PATCH_DIR}/diskhealth.wd"
    diskhealth_element=/srv/www2/app/View/Elements/diskhealth.ctp
    chown spui:root ${diskhealth_file}
    chmod 664 ${diskhealth_file}
    cp ${diskhealth_file} ${diskhealth_element}

    diskhealth_pane='echo $this->element( '\''diskhealth'\'' );'
    upgrades_page=/srv/www2/app/View/Upgrades/index.ctp
    
    # Determine if we need to inject the code into the web ui
    if ! grep -iq "diskhealth" ${upgrades_page} ; then
        inject_line=$(grep -n -m 1 '$this->end()' ${upgrades_page} | cut -d":" -f 1)
        sed -i "${inject_line}i${diskhealth_pane}" ${upgrades_page}
    fi
    exit 0
fi

exit 1
